@@ -1,43 +1,15 @@
 /* -*- Mode: C++; tab-width: 20; indent-tabs-mode: nil; c-basic-offset: 4 -*-
- * ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is Mozilla Foundation code.
- *
- * The Initial Developer of the Original Code is Mozilla Foundation.
- * Portions created by the Initial Developer are Copyright (C) 2005
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Vladimir Vukicevic <vladimir@pobox.com>
- *   Masayuki Nakano <masayuki@d-toybox.com>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+ * This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include <QPixmap>
-#include <QX11Info>
+#include <qglobal.h>
+#if (QT_VERSION < QT_VERSION_CHECK(5,0,0))
+#  include <QX11Info>
+#else
+#  include <QPlatformNativeInterface>
+#endif
 #include <QApplication>
 #include <QDesktopWidget>
 #include <QPaintEngine>
@@ -46,10 +18,13 @@
 
 #include "gfxFontconfigUtils.h"
 
+#include "mozilla/gfx/2D.h"
+
 #include "cairo.h"
 
 #include "gfxImageSurface.h"
 #include "gfxQPainterSurface.h"
+#include "nsUnicodeProperties.h"
 
 #ifdef MOZ_PANGO
 #include "gfxPangoFonts.h"
@@ -79,8 +54,14 @@
 #include "mozilla/Preferences.h"
 
 using namespace mozilla;
+using namespace mozilla::unicode;
+using namespace mozilla::gfx;
 
+#if (QT_VERSION < QT_VERSION_CHECK(5,0,0))
 #define DEFAULT_RENDER_MODE RENDER_DIRECT
+#else
+#define DEFAULT_RENDER_MODE RENDER_BUFFERED
+#endif
 
 static QPaintEngine::Type sDefaultQtPaintEngineType = QPaintEngine::Raster;
 gfxFontconfigUtils *gfxQtPlatform::sFontconfigUtils = nsnull;
@@ -91,6 +72,8 @@ static void do_qt_pixmap_unref (void *data)
     delete pmap;
 }
 
+static gfxImageFormat sOffscreenFormat = gfxASurface::ImageFormatRGB24;
+
 #ifndef MOZ_PANGO
 typedef nsDataHashtable<nsStringHashKey, nsRefPtr<FontFamily> > FontTable;
 typedef nsDataHashtable<nsCStringHashKey, nsTArray<nsRefPtr<FontEntry> > > PrefFontTable;
@@ -149,6 +132,9 @@ gfxQtPlatform::gfxQtPlatform()
     // Qt doesn't provide a public API to detect the graphicssystem type. We hack
     // around this by checking what type of graphicssystem a test QPixmap uses.
     QPixmap pixmap(1, 1);
+    if (pixmap.depth() == 16) {
+        sOffscreenFormat = gfxASurface::ImageFormatRGB16_565;
+    }
 #if (QT_VERSION < QT_VERSION_CHECK(4,8,0))
     if (pixmap.paintEngine())
         sDefaultQtPaintEngineType = pixmap.paintEngine()->type();
@@ -187,17 +173,47 @@ gfxQtPlatform::~gfxQtPlatform()
 #endif
 }
 
+#ifdef MOZ_X11
+Display*
+gfxQtPlatform::GetXDisplay(QWidget* aWindow)
+{
+#if (QT_VERSION < QT_VERSION_CHECK(5,0,0))
+#ifdef Q_WS_X11
+  return aWindow ? aWindow->x11Info().display() : QX11Info::display();
+#else
+  return nsnull;
+#endif
+#else
+  return (Display*)(qApp->platformNativeInterface()->
+    nativeResourceForWindow("display", aWindow ? aWindow->windowHandle() : nsnull));
+#endif
+}
+
+Screen*
+gfxQtPlatform::GetXScreen(QWidget* aWindow)
+{
+#if (QT_VERSION < QT_VERSION_CHECK(5,0,0))
+#ifdef Q_WS_X11
+  return ScreenOfDisplay(GetXDisplay(aWindow), aWindow ? aWindow->x11Info().screen() : QX11Info().screen());
+#else
+  return nsnull;
+#endif
+#else
+  return ScreenOfDisplay(GetXDisplay(aWindow),
+                         (int)qApp->platformNativeInterface()->
+                           nativeResourceForWindow("screen",
+                             aWindow ? aWindow->windowHandle() : nsnull));
+#endif
+}
+#endif
+
 already_AddRefed<gfxASurface>
 gfxQtPlatform::CreateOffscreenSurface(const gfxIntSize& size,
                                       gfxASurface::gfxContentType contentType)
 {
     nsRefPtr<gfxASurface> newSurface = nsnull;
 
-    // try to optimize it for 16bpp screen
-    gfxASurface::gfxImageFormat imageFormat = gfxASurface::FormatFromContent(contentType);
-    if (gfxASurface::CONTENT_COLOR == contentType) {
-      imageFormat = GetOffscreenFormat();
-    }
+    gfxASurface::gfxImageFormat imageFormat = OptimalFormatForContent(contentType);
 
 #ifdef CAIRO_HAS_QT_SURFACE
     if (mRenderMode == RENDER_QPAINTER) {
@@ -214,9 +230,9 @@ gfxQtPlatform::CreateOffscreenSurface(const gfxIntSize& size,
 
 #ifdef MOZ_X11
     XRenderPictFormat* xrenderFormat =
-        gfxXlibSurface::FindRenderFormat(QX11Info().display(), imageFormat);
+        gfxXlibSurface::FindRenderFormat(GetXDisplay(), imageFormat);
 
-    Screen* screen = ScreenOfDisplay(QX11Info().display(), QX11Info().screen());
+    Screen* screen = GetXScreen();
     newSurface = gfxXlibSurface::Create(screen, xrenderFormat, size);
 #endif
 
@@ -318,12 +334,12 @@ gfxQtPlatform::UpdateFontList()
             }
         }
 
-        fe->mItalic = PR_FALSE;
+        fe->mItalic = false;
         if (FcPatternGetInteger(fs->fonts[i], FC_SLANT, 0, &x) == FcResultMatch) {
             switch (x) {
             case FC_SLANT_ITALIC:
             case FC_SLANT_OBLIQUE:
-                fe->mItalic = PR_TRUE;
+                fe->mItalic = true;
             }
         }
 
@@ -345,7 +361,7 @@ nsresult
 gfxQtPlatform::ResolveFontName(const nsAString& aFontName,
                                 FontResolverCallback aCallback,
                                 void *aClosure,
-                                PRBool& aAborted)
+                                bool& aAborted)
 {
 #ifdef MOZ_PANGO
     return sFontconfigUtils->ResolveFontName(aFontName, aCallback,
@@ -458,7 +474,7 @@ gfxQtPlatform::MakePlatformFont(const gfxProxyFontEntry *aProxyEntry,
                                            aFontData, aLength);
 }
 
-PRBool
+bool
 gfxQtPlatform::IsFontFormatSupported(nsIURI *aFontURI, PRUint32 aFormatFlags)
 {
     // check for strange format flags
@@ -472,16 +488,16 @@ gfxQtPlatform::IsFontFormatSupported(nsIURI *aFontURI, PRUint32 aFormatFlags)
     if (aFormatFlags & (gfxUserFontSet::FLAG_FORMAT_WOFF     |
                         gfxUserFontSet::FLAG_FORMAT_OPENTYPE |
                         gfxUserFontSet::FLAG_FORMAT_TRUETYPE)) {
-        return PR_TRUE;
+        return true;
     }
 
     // reject all other formats, known and unknown
     if (aFormatFlags != 0) {
-        return PR_FALSE;
+        return false;
     }
 
     // no format hint set, need to look at data
-    return PR_TRUE;
+    return true;
 }
 #endif
 
@@ -526,7 +542,7 @@ FindFontForCharProc(nsStringHashKey::KeyType aKey,
                     nsRefPtr<FontFamily>& aFontFamily,
                     void* aUserArg)
 {
-    FontSearch *data = (FontSearch*)aUserArg;
+    GlobalFontMatch *data = (GlobalFontMatch*)aUserArg;
     aFontFamily->FindFontForChar(data);
     return PL_DHASH_NEXT;
 }
@@ -542,7 +558,8 @@ gfxQtPlatform::FindFontForChar(PRUint32 aCh, gfxFont *aFont)
         return nsnull;
     }
 
-    FontSearch data(aCh, aFont);
+    GlobalFontMatch data(aCh, GetScriptCode(aCh),
+                         (aFont ? aFont->GetStyle() : nsnull));
 
     // find fonts that support the character
     gPlatformFonts->Enumerate(FindFontForCharProc, &data);
@@ -561,7 +578,7 @@ gfxQtPlatform::FindFontForChar(PRUint32 aCh, gfxFont *aFont)
     return nsnull;
 }
 
-PRBool
+bool
 gfxQtPlatform::GetPrefFontEntries(const nsCString& aKey, nsTArray<nsRefPtr<gfxFontEntry> > *array)
 {
     return mPrefFonts.Get(aKey, array);
@@ -586,9 +603,13 @@ gfxQtPlatform::GetDPI()
 gfxImageFormat
 gfxQtPlatform::GetOffscreenFormat()
 {
-    if (qApp->desktop()->depth() == 16) {
-        return gfxASurface::ImageFormatRGB16_565;
-    }
-
-    return gfxASurface::ImageFormatRGB24;
+    return sOffscreenFormat;
 }
+
+bool
+gfxQtPlatform::SupportsAzure(BackendType& aBackend)
+{
+  aBackend = BACKEND_SKIA;
+  return true;
+}
+
