@@ -1,40 +1,7 @@
 /* -*- Mode: C++; tab-width: 20; indent-tabs-mode: nil; c-basic-offset: 4 -*-
- * ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is thebes gfx code.
- *
- * The Initial Developer of the Original Code is Mozilla Foundation.
- * Portions created by the Initial Developer are Copyright (C) 2008-2009
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   John Daggett <jdaggett@mozilla.com>
- *   Jonathan Kew <jfkthame@gmail.com>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+ * This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #ifdef MOZ_LOGGING
 #define FORCE_PR_LOG /* Allow logging in the release build */
@@ -46,6 +13,8 @@
 #include "nsReadableUtils.h"
 #include "nsUnicharUtils.h"
 #include "prlong.h"
+#include "nsNetUtil.h"
+#include "nsIProtocolHandler.h"
 
 #include "woff.h"
 
@@ -74,17 +43,19 @@ gfxProxyFontEntry::gfxProxyFontEntry(const nsTArray<gfxFontFaceSrc>& aFontFaceSr
              PRUint32 aLanguageOverride,
              gfxSparseBitSet *aUnicodeRanges)
     : gfxFontEntry(NS_LITERAL_STRING("Proxy"), aFamily),
-      mLoadingState(NOT_LOADING)
+      mLoadingState(NOT_LOADING),
+      mUnsupportedFormat(false),
+      mLoader(nsnull)
 {
-    mIsProxy = PR_TRUE;
+    mIsProxy = true;
     mSrcList = aFontFaceSrcList;
     mSrcIndex = 0;
     mWeight = aWeight;
     mStretch = aStretch;
-    mItalic = (aItalicStyle & (FONT_STYLE_ITALIC | FONT_STYLE_OBLIQUE)) != 0;
+    mItalic = (aItalicStyle & (NS_FONT_STYLE_ITALIC | NS_FONT_STYLE_OBLIQUE)) != 0;
     mFeatureSettings.AppendElements(aFeatureSettings);
     mLanguageOverride = aLanguageOverride;
-    mIsUserFont = PR_TRUE;
+    mIsUserFont = true;
 }
 
 gfxProxyFontEntry::~gfxProxyFontEntry()
@@ -92,7 +63,7 @@ gfxProxyFontEntry::~gfxProxyFontEntry()
 }
 
 gfxFont*
-gfxProxyFontEntry::CreateFontInstance(const gfxFontStyle *aFontStyle, PRBool aNeedsBold)
+gfxProxyFontEntry::CreateFontInstance(const gfxFontStyle *aFontStyle, bool aNeedsBold)
 {
     // cannot create an actual font for a proxy entry
     return nsnull;
@@ -114,7 +85,7 @@ gfxUserFontSet::AddFontFace(const nsAString& aFamilyName,
                             PRUint32 aWeight,
                             PRUint32 aStretch,
                             PRUint32 aItalicStyle,
-                            const nsString& aFeatureSettings,
+                            const nsTArray<gfxFontFeature>& aFeatureSettings,
                             const nsString& aLanguageOverride,
                             gfxSparseBitSet *aUnicodeRanges)
 {
@@ -123,10 +94,10 @@ gfxUserFontSet::AddFontFace(const nsAString& aFamilyName,
     nsAutoString key(aFamilyName);
     ToLowerCase(key);
 
-    PRBool found;
+    bool found;
 
     if (aWeight == 0)
-        aWeight = FONT_WEIGHT_NORMAL;
+        aWeight = NS_FONT_WEIGHT_NORMAL;
 
     // stretch, italic/oblique ==> zero implies normal
 
@@ -137,15 +108,12 @@ gfxUserFontSet::AddFontFace(const nsAString& aFamilyName,
     }
 
     // construct a new face and add it into the family
-    nsTArray<gfxFontFeature> featureSettings;
-    gfxFontStyle::ParseFontFeatureSettings(aFeatureSettings,
-                                           featureSettings);
     PRUint32 languageOverride =
         gfxFontStyle::ParseFontLanguageOverride(aLanguageOverride);
     proxyEntry =
         new gfxProxyFontEntry(aFontFaceSrcList, family, aWeight, aStretch,
                               aItalicStyle,
-                              featureSettings,
+                              aFeatureSettings,
                               languageOverride,
                               aUnicodeRanges);
     family->AddFontEntry(proxyEntry);
@@ -153,8 +121,8 @@ gfxUserFontSet::AddFontFace(const nsAString& aFamilyName,
     if (LOG_ENABLED()) {
         LOG(("userfonts (%p) added (%s) with style: %s weight: %d stretch: %d",
              this, NS_ConvertUTF16toUTF8(aFamilyName).get(),
-             (aItalicStyle & FONT_STYLE_ITALIC ? "italic" :
-                 (aItalicStyle & FONT_STYLE_OBLIQUE ? "oblique" : "normal")),
+             (aItalicStyle & NS_FONT_STYLE_ITALIC ? "italic" :
+                 (aItalicStyle & NS_FONT_STYLE_OBLIQUE ? "oblique" : "normal")),
              aWeight, aStretch));
     }
 #endif
@@ -169,7 +137,7 @@ gfxUserFontSet::AddFontFace(const nsAString& aFamilyName,
     nsAutoString key(aFamilyName);
     ToLowerCase(key);
 
-    PRBool found;
+    bool found;
 
     gfxMixedFontFamily *family = mFontFamilies.GetWeak(key, &found);
     if (!family) {
@@ -183,20 +151,20 @@ gfxUserFontSet::AddFontFace(const nsAString& aFamilyName,
 gfxFontEntry*
 gfxUserFontSet::FindFontEntry(const nsAString& aName, 
                               const gfxFontStyle& aFontStyle, 
-                              PRBool& aFoundFamily,
-                              PRBool& aNeedsBold,
-                              PRBool& aWaitForUserFont)
+                              bool& aFoundFamily,
+                              bool& aNeedsBold,
+                              bool& aWaitForUserFont)
 {
-    aWaitForUserFont = PR_FALSE;
+    aWaitForUserFont = false;
     gfxMixedFontFamily *family = GetFamily(aName);
 
     // no user font defined for this name
     if (!family) {
-        aFoundFamily = PR_FALSE;
+        aFoundFamily = false;
         return nsnull;
     }
 
-    aFoundFamily = PR_TRUE;
+    aFoundFamily = true;
     gfxFontEntry* fe = family->FindFontForStyle(aFontStyle, aNeedsBold);
 
     // if not a proxy, font has already been loaded
@@ -354,16 +322,60 @@ private:
     off_t        mOff;
 };
 
+#ifdef MOZ_OTS_REPORT_ERRORS
+struct OTSCallbackUserData {
+    gfxUserFontSet    *mFontSet;
+    gfxProxyFontEntry *mProxy;
+};
+
+/* static */ bool
+gfxUserFontSet::OTSMessage(void *aUserData, const char *format, ...)
+{
+    va_list va;
+    va_start(va, format);
+
+    // buf should be more than adequate for any message OTS generates,
+    // so we don't worry about checking the result of vsnprintf()
+    char buf[512];
+    (void)vsnprintf(buf, sizeof(buf), format, va);
+
+    va_end(va);
+
+    OTSCallbackUserData *d = static_cast<OTSCallbackUserData*>(aUserData);
+    d->mFontSet->LogMessage(d->mProxy, buf);
+
+    return false;
+}
+#endif
+
 // Call the OTS library to sanitize an sfnt before attempting to use it.
 // Returns a newly-allocated block, or NULL in case of fatal errors.
-static const PRUint8*
-SanitizeOpenTypeData(const PRUint8* aData, PRUint32 aLength,
-                     PRUint32& aSaneLength, bool aIsCompressed)
+const PRUint8*
+gfxUserFontSet::SanitizeOpenTypeData(gfxProxyFontEntry *aProxy,
+                                     const PRUint8* aData, PRUint32 aLength,
+                                     PRUint32& aSaneLength, bool aIsCompressed)
 {
     // limit output/expansion to 256MB
     ExpandingMemoryStream output(aIsCompressed ? aLength * 2 : aLength,
                                  1024 * 1024 * 256);
-    if (ots::Process(&output, aData, aLength)) {
+#ifdef MOZ_GRAPHITE
+#define PRESERVE_GRAPHITE true
+#else
+#define PRESERVE_GRAPHITE false
+#endif
+
+#ifdef MOZ_OTS_REPORT_ERRORS
+    OTSCallbackUserData userData;
+    userData.mFontSet = this;
+    userData.mProxy = aProxy;
+#define ERROR_REPORTING_ARGS &gfxUserFontSet::OTSMessage, &userData,
+#else
+#define ERROR_REPORTING_ARGS
+#endif
+
+    if (ots::Process(&output, aData, aLength,
+                     ERROR_REPORTING_ARGS
+                     PRESERVE_GRAPHITE)) {
         aSaneLength = output.Tell();
         return static_cast<PRUint8*>(output.forget());
     } else {
@@ -396,9 +408,27 @@ StoreUserFontData(gfxFontEntry* aFontEntry, gfxProxyFontEntry* aProxy,
     }
 }
 
-static void
-CopyWOFFMetadata(const PRUint8* aFontData, PRUint32 aLength,
-                 nsTArray<PRUint8>* aMetadata, PRUint32* aMetaOrigLen)
+struct WOFFHeader {
+    AutoSwap_PRUint32 signature;
+    AutoSwap_PRUint32 flavor;
+    AutoSwap_PRUint32 length;
+    AutoSwap_PRUint16 numTables;
+    AutoSwap_PRUint16 reserved;
+    AutoSwap_PRUint32 totalSfntSize;
+    AutoSwap_PRUint16 majorVersion;
+    AutoSwap_PRUint16 minorVersion;
+    AutoSwap_PRUint32 metaOffset;
+    AutoSwap_PRUint32 metaCompLen;
+    AutoSwap_PRUint32 metaOrigLen;
+    AutoSwap_PRUint32 privOffset;
+    AutoSwap_PRUint32 privLen;
+};
+
+void
+gfxUserFontSet::CopyWOFFMetadata(const PRUint8* aFontData,
+                                 PRUint32 aLength,
+                                 nsTArray<PRUint8>* aMetadata,
+                                 PRUint32* aMetaOrigLen)
 {
     // This function may be called with arbitrary, unvalidated "font" data
     // from @font-face, so it needs to be careful to bounds-check, etc.,
@@ -406,21 +436,6 @@ CopyWOFFMetadata(const PRUint8* aFontData, PRUint32 aLength,
     // This just saves a copy of the compressed data block; it does NOT check
     // that the block can be successfully decompressed, or that it contains
     // well-formed/valid XML metadata.
-    struct WOFFHeader {
-        AutoSwap_PRUint32 signature;
-        AutoSwap_PRUint32 flavor;
-        AutoSwap_PRUint32 length;
-        AutoSwap_PRUint16 numTables;
-        AutoSwap_PRUint16 reserved;
-        AutoSwap_PRUint32 totalSfntSize;
-        AutoSwap_PRUint16 majorVersion;
-        AutoSwap_PRUint16 minorVersion;
-        AutoSwap_PRUint32 metaOffset;
-        AutoSwap_PRUint32 metaCompLen;
-        AutoSwap_PRUint32 metaOrigLen;
-        AutoSwap_PRUint32 privOffset;
-        AutoSwap_PRUint32 privLen;
-    };
     if (aLength < sizeof(WOFFHeader)) {
         return;
     }
@@ -443,131 +458,25 @@ CopyWOFFMetadata(const PRUint8* aFontData, PRUint32 aLength,
 // This is called when a font download finishes.
 // Ownership of aFontData passes in here, and the font set must
 // ensure that it is eventually deleted via NS_Free().
-PRBool 
+bool
 gfxUserFontSet::OnLoadComplete(gfxProxyFontEntry *aProxy,
                                const PRUint8 *aFontData, PRUint32 aLength,
                                nsresult aDownloadStatus)
 {
+    // forget about the loader, as we no longer potentially need to cancel it
+    // if the entry is obsoleted
+    aProxy->mLoader = nsnull;
+
     // download successful, make platform font using font data
     if (NS_SUCCEEDED(aDownloadStatus)) {
-        gfxFontEntry *fe = nsnull;
-
-        gfxUserFontType fontType =
-            gfxFontUtils::DetermineFontDataType(aFontData, aLength);
-
-        // Save a copy of the metadata block (if present) for nsIDOMFontFace
-        // to use if required. Ownership of the metadata block will be passed
-        // to the gfxUserFontData record below.
-        // NOTE: after the non-OTS codepath using PrepareOpenTypeData is
-        // removed, we should defer this until after we've created the new
-        // fontEntry.
-        nsTArray<PRUint8> metadata;
-        PRUint32 metaOrigLen = 0;
-        if (fontType == GFX_USERFONT_WOFF) {
-            CopyWOFFMetadata(aFontData, aLength, &metadata, &metaOrigLen);
-        }
-
-        // Unwrap/decompress/sanitize or otherwise munge the downloaded data
-        // to make a usable sfnt structure.
-
-        // Because platform font activation code may replace the name table
-        // in the font with a synthetic one, we save the original name so that
-        // it can be reported via the nsIDOMFontFace API.
-        nsAutoString originalFullName;
-
-        if (gfxPlatform::GetPlatform()->SanitizeDownloadedFonts()) {
-           // Call the OTS sanitizer; this will also decode WOFF to sfnt
-            // if necessary. The original data in aFontData is left unchanged.
-            PRUint32 saneLen;
-            const PRUint8* saneData =
-                SanitizeOpenTypeData(aFontData, aLength, saneLen,
-                                     fontType == GFX_USERFONT_WOFF);
-            if (!saneData) {
-                LogMessage(aProxy, "rejected by sanitizer");
-            }
-            if (saneData) {
-                // The sanitizer ensures that we have a valid sfnt and a usable
-                // name table, so this should never fail unless we're out of
-                // memory, and GetFullNameFromSFNT is not directly exposed to
-                // arbitrary/malicious data from the web.
-                gfxFontUtils::GetFullNameFromSFNT(saneData, saneLen,
-                                                  originalFullName);
-                // Here ownership of saneData is passed to the platform,
-                // which will delete it when no longer required
-                fe = gfxPlatform::GetPlatform()->MakePlatformFont(aProxy,
-                                                                  saneData,
-                                                                  saneLen);
-                if (!fe) {
-                    LogMessage(aProxy, "not usable by platform");
-                }
-            }
-        } else {
-            // FIXME: this code can be removed once we remove the pref to
-            // disable the sanitizer; the PrepareOpenTypeData and
-            // ValidateSFNTHeaders functions will then be obsolete.
-            aFontData = PrepareOpenTypeData(aFontData, &aLength);
-
-            if (aFontData) {
-                if (gfxFontUtils::ValidateSFNTHeaders(aFontData, aLength)) {
-                    // ValidateSFNTHeaders has checked that we have a valid
-                    // sfnt structure and a usable 'name' table
-                    gfxFontUtils::GetFullNameFromSFNT(aFontData, aLength,
-                                                      originalFullName);
-                    // Here ownership of aFontData is passed to the platform,
-                    // which will delete it when no longer required
-                    fe = gfxPlatform::GetPlatform()->MakePlatformFont(aProxy,
-                                                                      aFontData,
-                                                                      aLength);
-                    if (!fe) {
-                        LogMessage(aProxy, "not usable by platform");
-                    }
-                    aFontData = nsnull; // we must NOT free this below!
-                } else {
-                    // the data was unusable, so just discard it
-                    // (error will be reported below, if logging is enabled)
-                    LogMessage(aProxy, "SFNT header or tables invalid");
-                }
-            }
-        }
-
-        if (aFontData) {
-            NS_Free((void*)aFontData);
-            aFontData = nsnull;
-        }
+        gfxFontEntry *fe = LoadFont(aProxy, aFontData, aLength);
+        aFontData = nsnull;
 
         if (fe) {
-            // copy OpenType feature/language settings from the proxy to the
-            // newly-created font entry
-            fe->mFeatureSettings.AppendElements(aProxy->mFeatureSettings);
-            fe->mLanguageOverride = aProxy->mLanguageOverride;
-            StoreUserFontData(fe, aProxy, originalFullName,
-                              &metadata, metaOrigLen);
-#ifdef PR_LOGGING
-            // must do this before ReplaceFontEntry() because that will
-            // clear the proxy's mFamily pointer!
-            if (LOG_ENABLED()) {
-                nsCAutoString fontURI;
-                aProxy->mSrcList[aProxy->mSrcIndex].mURI->GetSpec(fontURI);
-                LOG(("userfonts (%p) [src %d] loaded uri: (%s) for (%s) gen: %8.8x\n",
-                     this, aProxy->mSrcIndex, fontURI.get(),
-                     NS_ConvertUTF16toUTF8(aProxy->mFamily->Name()).get(),
-                     PRUint32(mGeneration)));
-            }
-#endif
-            ReplaceFontEntry(aProxy, fe);
             IncrementGeneration();
-            return PR_TRUE;
-        } else {
-#ifdef PR_LOGGING
-            if (LOG_ENABLED()) {
-                nsCAutoString fontURI;
-                aProxy->mSrcList[aProxy->mSrcIndex].mURI->GetSpec(fontURI);
-                LOG(("userfonts (%p) [src %d] failed uri: (%s) for (%s) error making platform font\n",
-                     this, aProxy->mSrcIndex, fontURI.get(),
-                     NS_ConvertUTF16toUTF8(aProxy->mFamily->Name()).get()));
-            }
-#endif
+            return true;
         }
+
     } else {
         // download failed
         LogMessage(aProxy, "download failed", nsIScriptError::errorFlag,
@@ -579,15 +488,14 @@ gfxUserFontSet::OnLoadComplete(gfxProxyFontEntry *aProxy,
     }
 
     // error occurred, load next src
-    LoadStatus status;
+    (void)LoadNext(aProxy);
 
-    status = LoadNext(aProxy);
-
-    // Even if loading failed, we need to bump the font-set generation
+    // We ignore the status returned by LoadNext();
+    // even if loading failed, we need to bump the font-set generation
     // and return true in order to trigger reflow, so that fallback
     // will be used where the text was "masked" by the pending download
     IncrementGeneration();
-    return PR_TRUE;
+    return true;
 }
 
 
@@ -601,6 +509,7 @@ gfxUserFontSet::LoadNext(gfxProxyFontEntry *aProxyEntry)
 
     if (aProxyEntry->mLoadingState == gfxProxyFontEntry::NOT_LOADING) {
         aProxyEntry->mLoadingState = gfxProxyFontEntry::LOADING_STARTED;
+        aProxyEntry->mUnsupportedFormat = false;
     } else {
         // we were already loading; move to the next source,
         // but don't reset state - if we've already timed out,
@@ -613,7 +522,7 @@ gfxUserFontSet::LoadNext(gfxProxyFontEntry *aProxyEntry)
     while (aProxyEntry->mSrcIndex < numSrc) {
         const gfxFontFaceSrc& currSrc = aProxyEntry->mSrcList[aProxyEntry->mSrcIndex];
 
-        // src local ==> lookup and load
+        // src local ==> lookup and load immediately
 
         if (currSrc.mIsLocal) {
             gfxFontEntry *fe =
@@ -642,32 +551,65 @@ gfxUserFontSet::LoadNext(gfxProxyFontEntry *aProxyEntry)
         else {
             if (gfxPlatform::GetPlatform()->IsFontFormatSupported(currSrc.mURI,
                     currSrc.mFormatFlags)) {
-                nsresult rv = StartLoad(aProxyEntry, &currSrc);
-                PRBool loadOK = NS_SUCCEEDED(rv);
-                if (loadOK) {
-#ifdef PR_LOGGING
-                    if (LOG_ENABLED()) {
-                        nsCAutoString fontURI;
-                        currSrc.mURI->GetSpec(fontURI);
-                        LOG(("userfonts (%p) [src %d] loading uri: (%s) for (%s)\n",
-                             this, aProxyEntry->mSrcIndex, fontURI.get(),
-                             NS_ConvertUTF16toUTF8(aProxyEntry->mFamily->Name()).get()));
+
+                nsresult rv;
+                bool loadDoesntSpin = false;
+                rv = NS_URIChainHasFlags(currSrc.mURI,
+                       nsIProtocolHandler::URI_SYNC_LOAD_IS_OK,
+                       &loadDoesntSpin);
+
+                if (NS_SUCCEEDED(rv) && loadDoesntSpin) {
+                    PRUint8 *buffer = nsnull;
+                    PRUint32 bufferLength = 0;
+
+                    // sync load font immediately
+                    rv = SyncLoadFontData(aProxyEntry, &currSrc, buffer,
+                                          bufferLength);
+
+                    if (NS_SUCCEEDED(rv) &&
+                        LoadFont(aProxyEntry, buffer, bufferLength)) {
+                        return STATUS_LOADED;
+                    } else {
+                        LogMessage(aProxyEntry, "font load failed",
+                                   nsIScriptError::errorFlag, rv);
                     }
-#endif
-                    return STATUS_LOADING;
+
                 } else {
-                    LogMessage(aProxyEntry, "download failed",
-                               nsIScriptError::errorFlag, rv);
+                    // otherwise load font async
+                    rv = StartLoad(aProxyEntry, &currSrc);
+                    bool loadOK = NS_SUCCEEDED(rv);
+
+                    if (loadOK) {
+#ifdef PR_LOGGING
+                        if (LOG_ENABLED()) {
+                            nsCAutoString fontURI;
+                            currSrc.mURI->GetSpec(fontURI);
+                            LOG(("userfonts (%p) [src %d] loading uri: (%s) for (%s)\n",
+                                 this, aProxyEntry->mSrcIndex, fontURI.get(),
+                                 NS_ConvertUTF16toUTF8(aProxyEntry->mFamily->Name()).get()));
+                        }
+#endif
+                        return STATUS_LOADING;
+                    } else {
+                        LogMessage(aProxyEntry, "download failed",
+                                   nsIScriptError::errorFlag, rv);
+                    }
                 }
             } else {
-                LogMessage(aProxyEntry, "format not supported",
-                           nsIScriptError::warningFlag);
+                // We don't log a warning to the web console yet,
+                // as another source may load successfully
+                aProxyEntry->mUnsupportedFormat = true;
             }
         }
 
         aProxyEntry->mSrcIndex++;
     }
 
+    if (aProxyEntry->mUnsupportedFormat) {
+        LogMessage(aProxyEntry, "no supported format found",
+                   nsIScriptError::warningFlag);
+    }
+
     // all src's failed; mark this entry as unusable (so fallback will occur)
     LOG(("userfonts (%p) failed all src for (%s)\n",
         this, NS_ConvertUTF16toUTF8(aProxyEntry->mFamily->Name()).get()));
@@ -687,6 +629,139 @@ gfxUserFontSet::IncrementGeneration()
 }
 
 
+gfxFontEntry*
+gfxUserFontSet::LoadFont(gfxProxyFontEntry *aProxy,
+                         const PRUint8 *aFontData, PRUint32 &aLength)
+{
+    // if the proxy doesn't belong to a family, we just bail as it won't be
+    // accessible/usable anyhow (maybe the font set got modified right as
+    // the load was completing?)
+    if (!aProxy->Family()) {
+        NS_Free(const_cast<PRUint8*>(aFontData));
+        return nsnull;
+    }
+
+    gfxFontEntry *fe = nsnull;
+
+    gfxUserFontType fontType =
+        gfxFontUtils::DetermineFontDataType(aFontData, aLength);
+
+    // Save a copy of the metadata block (if present) for nsIDOMFontFace
+    // to use if required. Ownership of the metadata block will be passed
+    // to the gfxUserFontData record below.
+    // NOTE: after the non-OTS codepath using PrepareOpenTypeData is
+    // removed, we should defer this until after we've created the new
+    // fontEntry.
+    nsTArray<PRUint8> metadata;
+    PRUint32 metaOrigLen = 0;
+    if (fontType == GFX_USERFONT_WOFF) {
+        CopyWOFFMetadata(aFontData, aLength, &metadata, &metaOrigLen);
+    }
+
+    // Unwrap/decompress/sanitize or otherwise munge the downloaded data
+    // to make a usable sfnt structure.
+
+    // Because platform font activation code may replace the name table
+    // in the font with a synthetic one, we save the original name so that
+    // it can be reported via the nsIDOMFontFace API.
+    nsAutoString originalFullName;
+
+    if (gfxPlatform::GetPlatform()->SanitizeDownloadedFonts()) {
+       // Call the OTS sanitizer; this will also decode WOFF to sfnt
+        // if necessary. The original data in aFontData is left unchanged.
+        PRUint32 saneLen;
+        const PRUint8* saneData =
+            SanitizeOpenTypeData(aProxy, aFontData, aLength, saneLen,
+                                 fontType == GFX_USERFONT_WOFF);
+        if (!saneData) {
+            LogMessage(aProxy, "rejected by sanitizer");
+        }
+        if (saneData) {
+            // The sanitizer ensures that we have a valid sfnt and a usable
+            // name table, so this should never fail unless we're out of
+            // memory, and GetFullNameFromSFNT is not directly exposed to
+            // arbitrary/malicious data from the web.
+            gfxFontUtils::GetFullNameFromSFNT(saneData, saneLen,
+                                              originalFullName);
+            // Here ownership of saneData is passed to the platform,
+            // which will delete it when no longer required
+            fe = gfxPlatform::GetPlatform()->MakePlatformFont(aProxy,
+                                                              saneData,
+                                                              saneLen);
+            if (!fe) {
+                LogMessage(aProxy, "not usable by platform");
+            }
+        }
+    } else {
+        // FIXME: this code can be removed once we remove the pref to
+        // disable the sanitizer; the PrepareOpenTypeData and
+        // ValidateSFNTHeaders functions will then be obsolete.
+        aFontData = PrepareOpenTypeData(aFontData, &aLength);
+
+        if (aFontData) {
+            if (gfxFontUtils::ValidateSFNTHeaders(aFontData, aLength)) {
+                // ValidateSFNTHeaders has checked that we have a valid
+                // sfnt structure and a usable 'name' table
+                gfxFontUtils::GetFullNameFromSFNT(aFontData, aLength,
+                                                  originalFullName);
+                // Here ownership of aFontData is passed to the platform,
+                // which will delete it when no longer required
+                fe = gfxPlatform::GetPlatform()->MakePlatformFont(aProxy,
+                                                                  aFontData,
+                                                                  aLength);
+                if (!fe) {
+                    LogMessage(aProxy, "not usable by platform");
+                }
+                aFontData = nsnull; // we must NOT free this!
+            } else {
+                // the data was unusable, so just discard it
+                // (error will be reported below, if logging is enabled)
+                LogMessage(aProxy, "SFNT header or tables invalid");
+            }
+        }
+    }
+
+    if (aFontData) {
+        NS_Free((void*)aFontData);
+        aFontData = nsnull;
+    }
+
+    if (fe) {
+        // copy OpenType feature/language settings from the proxy to the
+        // newly-created font entry
+        fe->mFeatureSettings.AppendElements(aProxy->mFeatureSettings);
+        fe->mLanguageOverride = aProxy->mLanguageOverride;
+        StoreUserFontData(fe, aProxy, originalFullName,
+                          &metadata, metaOrigLen);
+#ifdef PR_LOGGING
+        // must do this before ReplaceFontEntry() because that will
+        // clear the proxy's mFamily pointer!
+        if (LOG_ENABLED()) {
+            nsCAutoString fontURI;
+            aProxy->mSrcList[aProxy->mSrcIndex].mURI->GetSpec(fontURI);
+            LOG(("userfonts (%p) [src %d] loaded uri: (%s) for (%s) gen: %8.8x\n",
+                 this, aProxy->mSrcIndex, fontURI.get(),
+                 NS_ConvertUTF16toUTF8(aProxy->mFamily->Name()).get(),
+                 PRUint32(mGeneration)));
+        }
+#endif
+        ReplaceFontEntry(aProxy, fe);
+    } else {
+#ifdef PR_LOGGING
+        if (LOG_ENABLED()) {
+            nsCAutoString fontURI;
+            aProxy->mSrcList[aProxy->mSrcIndex].mURI->GetSpec(fontURI);
+            LOG(("userfonts (%p) [src %d] failed uri: (%s) for (%s)"
+                 " error making platform font\n",
+                 this, aProxy->mSrcIndex, fontURI.get(),
+                 NS_ConvertUTF16toUTF8(aProxy->mFamily->Name()).get()));
+        }
+#endif
+    }
+
+    return fe;
+}
+
 gfxMixedFontFamily*
 gfxUserFontSet::GetFamily(const nsAString& aFamilyName) const
 {
@@ -695,3 +770,4 @@ gfxUserFontSet::GetFamily(const nsAString& aFamilyName) const
 
     return mFontFamilies.GetWeak(key);
 }
+
